# BEGIN BPS TAGGED BLOCK {{{
# COPYRIGHT:
# 
# This software is Copyright (c) 2003-2008 Best Practical Solutions, LLC
#                                          <clkao@bestpractical.com>
# 
# (Except where explicitly superseded by other copyright notices)
# 
# 
# LICENSE:
# 
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of either:
# 
#   a) Version 2 of the GNU General Public License.  You should have
#      received a copy of the GNU General Public License along with this
#      program.  If not, write to the Free Software Foundation, Inc., 51
#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
#      their web page on the internet at
#      http://www.gnu.org/copyleft/gpl.html.
# 
#   b) Version 1 of Perl's "Artistic License".  You should have received
#      a copy of the Artistic License with this package, in the file
#      named "ARTISTIC".  The license is also available at
#      http://opensource.org/licenses/artistic-license.php.
# 
# This work is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# CONTRIBUTION SUBMISSION POLICY:
# 
# (The following paragraph is not intended to limit the rights granted
# to you to modify and distribute this software under the terms of the
# GNU General Public License and is only of importance to you if you
# choose to contribute your changes and enhancements to the community
# by submitting them to Best Practical Solutions, LLC.)
# 
# By intentionally submitting any modifications, corrections or
# derivatives to this work, or any other work intended for use with SVK,
# to Best Practical Solutions, LLC, you confirm that you are the
# copyright holder for those contributions and you grant Best Practical
# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
# perpetual, license to use, copy, create derivative works based on
# those contributions, and sublicense and distribute those contributions
# and any derivatives thereof.
# 
# END BPS TAGGED BLOCK }}}
package SVK::Editor::ByPass;

use base 'SVK::Editor';

__PACKAGE__->mk_accessors(qw(_editor));

# editor with the behaviour that when the subclass calls ->SUPER, it dispatches to ->{_editor}.

sub new {
    my $class = shift;
    my $self = ref($_[0]) =~ m/Editor/
	? $class->SUPER::new({ _editor => $_[0]})
	: $class->SUPER::new(@_);
    if ($self->_editor) {
	if (ref($self->_editor) eq 'ARRAY') {
	    Carp::cluck,die if $#{$self->_editor} == 1;
	    $self->_editor($self->_editor->[0]);
	}
    }
    Carp::cluck unless $self->_editor;
    return $self;
}

our $AUTOLOAD;
sub AUTOLOAD {
    my $self = shift;
    my $editor = $self->_editor;
    my $func = $AUTOLOAD;
    my $method;
    $func =~ s/.*:://;
    if ($self->_debug) {
	# XXX: debug hook instead of just warn?
	Carp::carp $editor.' '.$func.' '.join(',', @_)
    }
    return if $func =~ m/^[A-Z]/;
    $method = $editor->can($func);
    unless ($method) {
	# XXX: might be able to short circuit chained bypass
	# maybe autoload
	return $editor->$func(@_);
    }
    unshift @_, $editor;
    goto $method;
}

1;
